@@ -1,2 +1,2 @@
 # This is the one place where codeeditor depends on Pyzo itself
-from pyzo.util.qt import QtCore, QtGui, QtWidgets  # noqa
+from pyzo.qt import QtCore, QtGui, QtWidgets  # noqa
