@@ -1,8 +1,9 @@
-""" Module statusbar
+"""Module statusbar
 
 Functionality for status bar in pyzo.
 
 """
+
 from pyzo.qt import QtWidgets
 
 
@@ -26,21 +27,17 @@ class StatusBar(QtWidgets.QStatusBar):
         self.insertPermanentWidget(1, self.cursor_pos, 0)
 
     def updateCursorInfo(self, editor):
-
         # Get current line number
         nrow = 0
         ncol = 0
         if editor:
-            nrow = editor.textCursor().blockNumber()
-            nrow += 1  # is ln as in line number area
-            ncol = editor.textCursor().positionInBlock()
-            ncol += 1
+            nrow = editor.textCursor().blockNumber() + 1
+            ncol = editor.textCursor().positionInBlock() + 1
 
-        position_txt = "Line: {}, Column: {} ".format(str(nrow), str(ncol))
+        position_txt = "Line: {}, Column: {} ".format(nrow, ncol)
         self.cursor_pos.setText(position_txt)
 
     def updateFileEncodingInfo(self, editor):
-
         fe_txt = ""
         if editor:
             fe_txt = editor.encoding.upper()
