@@ -1,9 +1,3 @@
-# -*- coding: utf-8 -*-
-# Copyright (C) 2016, the Pyzo development team
-#
-# Pyzo is distributed under the terms of the 2-Clause BSD License.
-# The full license can be found in 'license.txt'.
-
 import os
 import sys
 import time
@@ -18,11 +12,27 @@ class Debugger(bdb.Bdb):
         self._wait_for_mainpyfile = False  # from pdb, do we need this?
         bdb.Bdb.__init__(self)
         self._debugmode = 0  # 0: no debug,  1: postmortem,  2: full debug
-        self._files_with_offset = []
+        self._files_with_offset = set()
+        if hasattr(sys, "breakpointhook"):
+            self._original_breakpointhook = sys.breakpointhook
+            sys.breakpointhook = self.custom_breakpointhook
+        self._last_db_command = None
+
+    def custom_breakpointhook(self, *args, **kwargs):
+        """handle "breakpoint()" commands
+        see https://docs.python.org/3/library/sys.html#sys.breakpointhook
+        """
+        s = os.environ.get("PYTHONBREAKPOINT", None)
+        if s == "0":
+            pass  # deactivated
+        elif s is None or s == "":
+            self.set_trace()
+        else:
+            self._original_breakpointhook(*args, **kwargs)
 
     def clear_all_breaks(self):
         bdb.Bdb.clear_all_breaks(self)
-        self._files_with_offset = []
+        self._files_with_offset.clear()
 
     def trace_dispatch(self, frame, event, arg):
         # Overload to deal with offset in filenames
@@ -37,7 +47,7 @@ class Debugger(bdb.Bdb):
                 offset = None
             if offset is not None:
                 # This is a cell or selected lines being executed
-                self._files_with_offset.append(ori_filename)
+                self._files_with_offset.add(ori_filename)
                 if clean_filename.startswith("<"):
                     self.fncache[ori_filename] = ori_filename
                 for i in self.breaks.get(clean_filename, []):
@@ -45,16 +55,26 @@ class Debugger(bdb.Bdb):
 
         return bdb.Bdb.trace_dispatch(self, frame, event, arg)
 
-    def interaction(self, frame, traceback=None, pm=False):
+    def interaction(self, frame, traceback=None, pm=False, framesBefore=None):
         """Enter an interaction-loop for debugging. No GUI events are
         processed here. We leave this event loop at some point, after
-        which the conrol flow will proceed.
+        which the control flow will proceed.
 
         This is called to enter debug-mode at a breakpoint, or to enter
         post-mortem debugging.
+
+        Special case for frames where f_back is None, e.g. in a generator expression:
+            For post mortem debugging we will allow to continue at the
+            frame before, e.g. before the generator expression.
+            see https://stackoverflow.com/a/51867843
         """
         interpreter = sys._pyzoInterpreter
 
+        if framesBefore is None:
+            framesBefore = []
+        else:
+            framesBefore = framesBefore[:]
+
         # Collect frames
         frames = []
         while frame:
@@ -66,7 +86,10 @@ class Debugger(bdb.Bdb):
             if "interactiveshell.py" in co_filename:
                 break  # IPython kernel
             frames.insert(0, frame)
-            frame = frame.f_back
+            if frame.f_back is None and pm and len(framesBefore) > 0:
+                frame = framesBefore.pop()
+            else:
+                frame = frame.f_back
 
         # Tell interpreter our stack
         if frames:
@@ -88,11 +111,14 @@ class Debugger(bdb.Bdb):
         # Enter interact loop. We may hang in here for a while ...
         self._interacting = True
         while self._interacting:
-            time.sleep(0.05)
-            interpreter.process_commands()
-            pe = os.getenv("PYZO_PROCESS_EVENTS_WHILE_DEBUGGING", "").lower()
-            if pe in ("1", "true", "yes"):
-                interpreter.guiApp.process_events()
+            try:
+                time.sleep(0.05)
+                interpreter.process_commands()
+                pe = os.getenv("PYZO_PROCESS_EVENTS_WHILE_DEBUGGING", "").lower()
+                if pe in ("1", "true", "yes"):
+                    interpreter.guiApp.process_events()
+            except KeyboardInterrupt:
+                self.message("KeyboardInterrupt")
 
         # Reset
         self._debugmode = 0
@@ -104,6 +130,7 @@ class Debugger(bdb.Bdb):
     def stopinteraction(self):
         """Stop the interaction loop."""
         self._interacting = False
+        sys.settrace(None)
 
     def set_on(self):
         """To turn debugging on right before executing code."""
@@ -117,11 +144,7 @@ class Debugger(bdb.Bdb):
         self.returnframe = None
         self.quitting = False
         self.stoplineno = -1
-        # Set tracing or not
-        if self.breaks:
-            sys.settrace(self.trace_dispatch)
-        else:
-            sys.settrace(None)
+        self._last_db_command = None
 
     def message(self, msg):
         """Alias for interpreter.write(), but appends a newline.
@@ -181,7 +204,7 @@ class Debugger(bdb.Bdb):
             )
 
     def do_clear(self, arg):
-        """ """
+        """ """  # this docstring is a single space on purpose
         # Clear breakpoints, we need to overload from Bdb,
         # but do not expose this command to the user.
         """cl(ear) filename:lineno\ncl(ear) [bpnumber [bpnumber...]]
@@ -272,12 +295,12 @@ class Debugger(bdb.Bdb):
         for name in dir(self):
             if name.startswith("do_"):
                 doc = getattr(self, name).__doc__
-                if doc:
+                if doc and doc != " ":
                     docs[name[3:]] = doc.strip()
 
         if not arg:
             print("All debug commands:")
-            # Show docs in  order
+            # Show docs in order
             for name in [
                 "start",
                 "stop",
@@ -287,6 +310,7 @@ class Debugger(bdb.Bdb):
                 "next",
                 "step",
                 "return",
+                "jump",
                 "continue",
                 "where",
                 "events",
@@ -309,18 +333,143 @@ class Debugger(bdb.Bdb):
             else:
                 print("Unknown debug command: %s" % name)
 
-    def do_start(self, arg):
-        """Start postmortem debugging from the last uncaught exception."""
+    def _get_traceback_combinations(self):
+        """get all possible combinations for tracebacks
 
-        # Get traceback
+        If the exception is an ExceptionGroup, it can consist of sub-exeptions
+        and even nested ExceptionGroup objects. We identify a branch via the
+        sub-exception numbers that are printed in the traceback ("---- 1 ----" etc.).
+
+        A tuple (3, 2) addresses the ValueError sub-sub-exception in the following example.
+        ExceptionGroup  --> ()
+            ---- 1 ----  --> (1,)
+            ZeroDivisionError
+            ---- 2 ----  --> (2,)
+            KeyError
+            ---- 3 ----  --> (3,)
+            ExceptionGroup
+                ---- 1 ----  --> (3, 1)
+                TypeError
+                ---- 2 ----  --> (3, 2)
+                ValueError
+
+        return value: tb_combs, first_nongroup
+            tb_combs ... a dictionary of kv-pairs
+                key: the tuple branch-identifier
+                value: the traceback of that (sub-)exception
+            first_nongroup ... the key to the tb_combs dict to the first traceback of an
+                non-ExceptionGroup -- useful as a default selection
+        """
+        tb_combs = {}
+        first_nongroup = None
         try:
+            value = sys.last_value
+            # Python 2.7 exceptions have no __traceback__ attribute
             tb = sys.last_traceback
         except AttributeError:
             tb = None
 
+        if tb is not None:
+            # add all combinations for nested exception groups
+            p = ()
+            stack = [(value, p)]
+            nongroup_exceptions = []
+            tb_combs[p] = tb
+            while len(stack) > 0:
+                e, p = stack.pop()
+                if e.__class__.__name__ == "ExceptionGroup":
+                    for num, e2 in enumerate(e.exceptions, 1):
+                        p2 = p + (num,)
+                        stack.append((e2, p2))
+                        tb_combs[p2] = e2.__traceback__
+                else:
+                    nongroup_exceptions.append(p)
+            if nongroup_exceptions:
+                first_nongroup = min(nongroup_exceptions)
+        return tb_combs, first_nongroup
+
+    def _get_selected_traceback(self, arg):
+        """get the current traceback, and in case of ExceptionGroup, select one of many"""
+        tb_combs, first_nongroup = self._get_traceback_combinations()
+        tb = tb_combs.get((), None)
+        if len(tb_combs) > 1:
+            # there is at least one ExceptionGroup
+            user_selected_comb = None
+            selected_comb = None
+            if arg != "":
+                try:
+                    user_selected_comb = tuple([int(s) for s in arg.split()])
+                    # Selecting the root-ExceptionGroup would normally be done with
+                    # an empty tuple. But we want the empty tuple, which is args == "",
+                    # to be the "automatic" selection option.
+                    # So we use "DB START 0" to get the root-ExceptionGroup
+                    # and "DB START" to automatically select the traceback.
+                    if user_selected_comb == (0,):
+                        user_selected_comb = ()
+                    if user_selected_comb not in tb_combs:
+                        user_selected_comb = None
+                except Exception:
+                    user_selected_comb = None
+
+                if user_selected_comb is None:
+                    self.message(
+                        "invalid parameter " + repr(arg) + " -- will be ignored"
+                    )
+                else:
+                    selected_comb = user_selected_comb
+
+            if selected_comb is None:
+                # automatically select the traceback
+                if first_nongroup is not None:
+                    selected_comb = first_nongroup
+                else:
+                    selected_comb = ()
+
+            tb = tb_combs[selected_comb]
+
+            if user_selected_comb is None:
+                cw = 16
+                msg_lines = [
+                    "There is at least one ExceptionGroup. To select the traceback of a specific group,",
+                    "stop the debugger and type the magic command DB START with (nested) group numbers.",
+                    "Possible commands for the last traceback are:",
+                    "db stop".ljust(cw) + "<-- stops the debugger",
+                    "db traceback".ljust(cw) + "<-- prints the last traceback again",
+                    "db start".ljust(cw)
+                    + "<-- this will automatically choose the first non-ExceptionGroup",
+                ]
+                for comb in sorted(tb_combs.keys()):
+                    comment = ""
+                    if comb == selected_comb:
+                        if selected_comb == first_nongroup:
+                            comment = "<-- currently used (automatically chosen -- first non-exc-group)"
+                        else:
+                            comment = (
+                                "<-- currently used (automatically chosen -- fallback)"
+                            )
+
+                    if comb == ():
+                        comb_string = "0"
+                        if comment == "":
+                            comment = "<-- zero means: explicitly choose the root ExceptionGroup"
+                    else:
+                        comb_string = " ".join([str(a) for a in comb])
+                    msg_lines.append(("db start " + comb_string).ljust(cw) + comment)
+                self.message("\n".join(msg_lines))
+        return tb
+
+    def do_start(self, arg):
+        """Start postmortem debugging from the last uncaught exception."""
+
+        # Since the introduction of ExceptionGroup, there could be multiple tracebacks
+        # of interest. A specific one can be selected using the "arg" parameter.
+        tb = self._get_selected_traceback(arg)
+
         # Get top frame
         frame = None
+        framesBefore = []
         while tb:
+            framesBefore.append(frame)
             frame = tb.tb_frame
             tb = tb.tb_next
 
@@ -328,7 +477,7 @@ class Debugger(bdb.Bdb):
         if self._debugmode:
             self.message("Already in debug mode.")
         elif frame:
-            self.interaction(frame, None, pm=True)
+            self.interaction(frame, None, pm=True, framesBefore=framesBefore)
         else:
             self.message("No debug information available.")
 
@@ -427,6 +576,23 @@ class Debugger(bdb.Bdb):
         elif self._debugmode == 1:
             self.message("Cannot use 'continue' in postmortem debug mode.")
         else:
+            # the following line is a workaround for a specific problem:
+            sys._pyzoInterpreter.apply_breakpoints()
+
+            # run the following code without any breakpoints:
+            """
+            [breakpoint() for _ in range(1)]
+            # [breakpoint()]  # this does not cause any problem
+            x = 1
+            y = 2  # set a breakpoint in this line once stopped in the upper breakpoint()
+            """
+            # The breakpoint at the line above would be ignored if:
+            # a) the interruption is inside a list comprehension
+            # and b) the Python interpreter version is >= 3.0 and < 3.12
+            # The problem would also occur when pausing inside a list comprehension.
+            # As a workaround we call apply_breakpoints a second time, see above.
+
+            self._last_db_command = "continue"
             self.set_continue()
             self.stopinteraction()
 
@@ -437,6 +603,7 @@ class Debugger(bdb.Bdb):
         elif self._debugmode == 1:
             self.message("Cannot use 'step' in postmortem debug mode.")
         else:
+            self._last_db_command = "step"
             self.set_step()
             self.stopinteraction()
 
@@ -450,6 +617,7 @@ class Debugger(bdb.Bdb):
             self.message("Cannot use 'next' in postmortem debug mode.")
         else:
             frame = interpreter._dbFrames[-1]
+            self._last_db_command = "next"
             self.set_next(frame)
             self.stopinteraction()
 
@@ -462,11 +630,42 @@ class Debugger(bdb.Bdb):
         elif self._debugmode == 1:
             self.message("Cannot use 'return' in postmortem debug mode.")
         else:
+            self._last_db_command = "return"
             frame = interpreter._dbFrames[-1]
             self.set_return(frame)
             self.stopinteraction()
 
+    def do_jump(self, arg):
+        """Jump to a specific line, not executing code in between."""
+        interpreter = sys._pyzoInterpreter
+
+        if self._debugmode == 0:
+            self.message("Not in debug mode.")
+        elif self._debugmode == 1:
+            self.message("Cannot use 'jump' in postmortem debug mode.")
+        elif interpreter._dbFrameIndex != len(interpreter._dbFrames):
+            self.message(
+                "Jumping is only possible inside the bottom frame (highest index)."
+            )
+        else:
+            frame = interpreter._dbFrames[interpreter._dbFrameIndex - 1]
+            try:
+                lineno = int(arg)
+                f = interpreter._dbFrames[-1]
+                offset = interpreter.correctfilenameandlineno(f.f_code.co_filename, 0)[
+                    1
+                ]
+                frame.f_lineno = lineno - offset
+            except ValueError as e:
+                self.message("Error DB JUMP: " + str(e))
+            self.writestatus()
+
     def do_events(self, arg):
         """Process GUI events for the integrated GUI toolkit."""
         interpreter = sys._pyzoInterpreter
         interpreter.guiApp.process_events()
+
+    def do_traceback(self, arg):
+        """Print the last traceback (again)."""
+        interpreter = sys._pyzoInterpreter
+        interpreter.showtraceback(True)
