@@ -1,9 +1,3 @@
-# -*- coding: utf-8 -*-
-# Copyright (C) 2016, the Pyzo development team
-#
-# Pyzo is distributed under the terms of the 2-Clause BSD License.
-# The full license can be found in 'license.txt'.
-
 """
 Module to integrate GUI event loops in the Pyzo interpreter.
 
@@ -163,6 +157,7 @@ class App_asyncio_new(App_base):
                     except KeyboardInterrupt:
                         self._keyboard_interrupt()
                         continue
+
         if not hasattr(new_loop, "original_run_forever"):
             new_loop.original_run_forever = new_loop.run_forever
         new_loop.run_forever = new_run_forever
@@ -492,7 +487,7 @@ class App_qt(App_base):
                         if hash(val) == hash(getattr(theApp.__class__, key)):
                             continue
                     # Make method?
-                    if hasattr(val, "__call__"):
+                    if callable(val):
                         if hasattr(val, "im_func"):
                             val = val.im_func  # Python 2.x
                         val = types.MethodType(val, theApp.__class__)
@@ -502,7 +497,10 @@ class App_qt(App_base):
                     except Exception:
                         pass  # tough luck
 
-                setattr(theApp, "exec", theApp.exec_)
+                # "exec" as an attribute would cause a syntax error in Python 2.7
+                # because it is a keyword.
+                # So we have to use setattr instead of assigning theApp.exec
+                setattr(theApp, "exec", theApp.exec_)  # noqa: B010
 
                 # Call init function (in case the user overloaded it)
                 theApp.__init__(*args, **kwargs)
@@ -524,7 +522,8 @@ class App_qt(App_base):
                 # Store local namespaces (scopes) of any functions that
                 # precede this call. It might have a widget or application
                 # object that should not be deleted ...
-                import inspect, __main__
+                import inspect
+                import __main__
 
                 for caller in inspect.stack()[1:]:
                     frame, name = caller[0], caller[3]
@@ -687,9 +686,9 @@ class App_wx(App_base):
         orig_mainloop = None
         if ver[:3] >= "2.5":
             if hasattr(wx, "_core_"):
-                core = getattr(wx, "_core_")
+                core = wx._core_
             elif hasattr(wx, "_core"):
-                core = getattr(wx, "_core")
+                core = wx._core
             else:
                 raise ImportError
             orig_mainloop = core.PyApp_MainLoop
