@@ -1,5 +1,3 @@
-# Copyright (C) 2013 Almar Klein
-
 """
 This module defines file system proxies to be used for the file browser.
 For now, there is only one: the native file system. But in time,
@@ -32,7 +30,7 @@ class Task:
     obtain the result (or raise an error).
     """
 
-    __slots__ = ["_params", "_result", "_error"]
+    __slots__ = ["_error", "_params", "_result"]
 
     def __init__(self, **params):
         if not params:
@@ -427,7 +425,7 @@ class NativeFSProxy(BaseFSProxy):
 
     def rename(self, path1, path2):
         if op.exists(path2):
-            raise FileExistsError("file {!r} already exists".format(path))
+            raise FileExistsError("file {!r} already exists".format(path2))
         os.rename(path1, path2)
 
     def remove(self, path):
