@@ -1,15 +1,10 @@
-# -*- coding: utf-8 -*-
-# Copyright (C) 2013, the Pyzo development team
-#
-# Yoton is distributed under the terms of the 2-Clause BSD License.
-# The full license can be found in 'license.txt'.
-
-""" Module yoton.channels.channels_reqprep
+"""Module yoton.channels.channels_reqprep
 
 Defines the channel classes for the req/rep pattern.
 
 """
 
+import sys
 import time
 import threading
 
@@ -76,11 +71,7 @@ class Future(object):
         self._auto_cancel_timeout = 10.0
 
     def _send(self, msg):
-        """_send(msg)
-
-        For sending pre-request messages 'req?', 'req-'.
-
-        """
+        """For sending pre-request messages 'req?', 'req-'."""
         msg = msg.encode("utf-8")
         try:
             self._req_channel._send(msg, 0, self._request_id + REQREP_SEQ_REF)
@@ -90,13 +81,10 @@ class Future(object):
             self.cancel()
 
     def _resend_if_necessary(self):
-        """_resend_if_necessary()
-
-        Resends the pre-request message if we have not done so for the last
+        """Resends the pre-request message if we have not done so for the last
         0.5 second.
 
-        This will also auto-cancel the message if it is resend over 20 times.
-
+        This will also auto-cancel the message if it is resent over 20 times.
         """
         timetime = time.time()
         if self._status != 0:
@@ -108,13 +96,10 @@ class Future(object):
             self._next_send_time = timetime + 0.5
 
     def set_auto_cancel_timeout(self, timeout):
-        """set_auto_cancel_timeout(timeout):
-
-        Set the timeout after which the call is automatically cancelled
+        """Set the timeout after which the call is automatically cancelled
         if it is not done yet. By default, this value is 10 seconds.
 
         If timeout is None, there is no limit to the wait time.
-
         """
         if timeout is None:
             timeout = 999999999999999999.0
@@ -124,12 +109,9 @@ class Future(object):
             raise ValueError("A timeout cannot be negative")
 
     def cancel(self):
-        """cancel()
-
-        Attempt to cancel the call. If the call is currently being executed
+        """Attempt to cancel the call. If the call is currently being executed
         and cannot be cancelled then the method will return False, otherwise
         the call will be cancelled and the method will return True.
-
         """
 
         if self._status == 1:
@@ -147,56 +129,38 @@ class Future(object):
             return True
 
     def cancelled(self):
-        """cancelled()
-
-        Return True if the call was successfully cancelled.
-
-        """
+        """Return True if the call was successfully cancelled."""
         return self._status == 2
 
     def running(self):
-        """running()
-
-        Return True if the call is currently being executed and cannot be
-        cancelled.
-
-        """
+        """Return True if the call is currently being executed and cannot be cancelled."""
         return self._status == 1
 
     def done(self):
-        """done()
-
-        Return True if the call was successfully cancelled or finished running.
-
-        """
+        """Return True if the call was successfully cancelled or finished running."""
         return self._status in [2, 3, 4]
 
     def _wait(self, timeout):
-        """_wait(timeout)
-
-        Wait for the request to be handled for the specified amount of time.
+        """Wait for the request to be handled for the specified amount of time.
         While waiting, the ReqChannel local event loop is called so that
         pre-request messages can be exchanged.
-
         """
 
-        # No timout means a veeeery long timeout
+        # No timeout means a veeeery long timeout
         if timeout is None:
             timeout = 999999999999999999.0
 
-        # Receive packages untill we receive the one we want,
-        # or untill time runs out
+        # Receive packages until we receive the one we want,
+        # or until time runs out
         timestamp = time.time() + timeout
         while (self._status < 2) and (time.time() < timestamp):
             self._req_channel._process_events_local()
             time.sleep(0.01)  # 10 ms
 
     def result(self, timeout=None):
-        """result(timeout=None)
-
-        Return the value returned by the call. If the call hasn’t yet
+        """Return the value returned by the call. If the call hasn't yet
         completed then this method will wait up to timeout seconds. If
-        the call hasn’t completed in timeout seconds, then a TimeoutError
+        the call hasn't completed in timeout seconds, then a TimeoutError
         will be raised. timeout can be an int or float. If timeout is not
         specified or None, there is no limit to the wait time.
 
@@ -204,7 +168,6 @@ class Future(object):
         will be raised.
 
         If the call raised, this method will raise the same exception.
-
         """
 
         # Wait
@@ -221,13 +184,10 @@ class Future(object):
             return self._result
 
     def result_or_cancel(self, timeout=1.0):
-        """result_or_cancel(timeout=1.0)
-
-        Return the value returned by the call. If the call hasn’t yet
+        """Return the value returned by the call. If the call hasn't yet
         completed then this method will wait up to timeout seconds. If
-        the call hasn’t completed in timeout seconds, then the call is
+        the call hasn't completed in timeout seconds, then the call is
         cancelled and the method will return None.
-
         """
 
         # Wait
@@ -241,11 +201,9 @@ class Future(object):
             return None
 
     def exception(self, timeout=None):
-        """exception(timeout)
-
-        Return the exception raised by the call. If the call hasn’t yet
+        """Return the exception raised by the call. If the call hasn't yet
         completed then this method will wait up to timeout seconds. If
-        the call hasn’t completed in timeout seconds, then a TimeoutError
+        the call hasn't completed in timeout seconds, then a TimeoutError
         will be raised. timeout can be an int or float. If timeout is not
         specified or None, there is no limit to the wait time.
 
@@ -253,7 +211,6 @@ class Future(object):
         will be raised.
 
         If the call completed without raising, None is returned.
-
         """
 
         # Wait
@@ -270,9 +227,7 @@ class Future(object):
             return None  # no exception
 
     def add_done_callback(self, fn):
-        """add_done_callback(fn)
-
-        Attaches the callable fn to the future. fn will be called, with
+        """Attaches the callable fn to the future. fn will be called, with
         the future as its only argument, when the future is cancelled or
         finishes running.
 
@@ -283,11 +238,10 @@ class Future(object):
 
         If the future has already completed or been cancelled, fn will be
         called immediately.
-
         """
 
         # Check
-        if not hasattr(fn, "__call__"):
+        if not callable(fn):
             raise ValueError("add_done_callback expects a callable.")
 
         # Add
@@ -297,9 +251,7 @@ class Future(object):
             self._callbacks.append(fn)
 
     def set_running_or_notify_cancel(self):
-        """set_running_or_notify_cancel()
-
-        This method should only be called by Executor implementations before
+        """This method should only be called by Executor implementations before
         executing the work associated with the Future and by unit tests.
 
         If the method returns False then the Future was cancelled, i.e.
@@ -311,7 +263,6 @@ class Future(object):
 
         This method can only be called once and cannot be called after
         Future.set_result() or Future.set_exception() have been called.
-
         """
 
         if self._status == 2:
@@ -325,12 +276,9 @@ class Future(object):
             )
 
     def set_result(self, result):
-        """set_result(result)
-
-        Sets the result of the work associated with the Future to result.
+        """Sets the result of the work associated with the Future to result.
         This method should only be used by Executor implementations and
         unit tests.
-
         """
 
         # Set result if indeed in running state
@@ -341,12 +289,9 @@ class Future(object):
                 yoton.call_later(fn, 0, self)
 
     def set_exception(self, exception):
-        """set_exception(exception)
-
-        Sets the result of the work associated with the Future to the
+        """Sets the result of the work associated with the Future to the
         Exception exception. This method should only be used by Executor
         implementations and unit tests.
-
         """
 
         # Check
@@ -412,7 +357,7 @@ class ReqChannel(BaseChannel):
     # Notes on load balancing:
     #
     # Firstly, each request has an id. Which is an integer number
-    # which is increased at each new request. The id is send via
+    # which is increased at each new request. The id is sent via
     # the dest_seq. For pre-request messages an offset is added
     # to recognize these meta-messages.
     #
@@ -487,14 +432,11 @@ class ReqChannel(BaseChannel):
             return proxy_function
 
     def _handle_request(self, name, *args, **kwargs):
-        """_handle_request(request, callback, **kwargs)
+        """Post a request. This creates a Future instance and stores
+        it. A message is sent asking any repliers to respond.
 
-        Post a request. This creates a Future instance and stores
-        it. A message is send asking any repliers to respond.
-
-        The actual request will be send when a reply to our pre-request
+        The actual request will be sent when a reply to our pre-request
         is received. This all hapens in the yoton event loop.
-
         """
 
         # Create request object
@@ -517,17 +459,14 @@ class ReqChannel(BaseChannel):
         return item
 
     def _resend_requests(self):
-        """_resend_requests()
-
-        See if we should resend our older requests. Periodically calling
+        """See if we should resend our older requests. Periodically calling
         this method enables doing a request while the replier is not yet
         attached to the network.
 
         This also allows the Future objects to cancel themselves if it
         takes too long.
-
         """
-        for request_id in [key for key in self._request_items.keys()]:
+        for request_id in list(self._request_items.keys()):
             item = self._request_items[request_id]
             # Remove items that are really old
             if item.cancelled():
@@ -536,15 +475,12 @@ class ReqChannel(BaseChannel):
                 item._resend_if_necessary()
 
     def _recv_item(self):
-        """_recv_item()
-
-        Receive item. If a reply is send that is an acknowledgement
+        """Receive item. If a reply is sent that is an acknowledgement
         of a replier that it wants to handle our request, the
-        correpsonding request is send to that replier.
+        correpsonding request is sent to that replier.
 
         This is a kind of mini-event loop thingy that should be
         called periodically to keep things going.
-
         """
 
         # Receive package
@@ -596,11 +532,7 @@ class ReqChannel(BaseChannel):
                 return item
 
     def _process_events_local(self, dummy=None):
-        """_process_events_local()
-
-        Process events only for this object. Used by _handle_now().
-
-        """
+        """Process events only for this object. Used by _handle_now()."""
 
         # Check periodically if we should resend (or clean up) old requests
         if time.time() > self._next_recheck_time:
@@ -627,7 +559,7 @@ class RepChannel(BaseChannel):
 
     The reply part of the request/reply messaging pattern.
     A RepChannel instance receives request and sends the corresponding
-    replies. The requests are send from a yoton.ReqChannel instance.
+    replies. The requests are sent from a yoton.ReqChannel instance.
 
     This class adopts req/rep in a remote procedure call (RPC) scheme.
 
@@ -677,15 +609,12 @@ class RepChannel(BaseChannel):
     # Node that setters for normal and event_driven mode are specified in
     # channels_base.py
     def set_mode(self, mode):
-        """set_mode(mode)
-
-        Set the replier to its operating mode, or turn it off.
+        """Set the replier to its operating mode, or turn it off.
 
         Modes:
           * 0 or 'off': do not process requests
           * 1 or 'event': use the yoton event loop to process requests
           * 2 or 'thread': process requests in a separate thread
-
         """
 
         if isinstance(mode, basestring):
@@ -705,12 +634,9 @@ class RepChannel(BaseChannel):
             raise ValueError("Invalid mode for ReqChannel instance.")
 
     def _handle_request(self, message):
-        """_handle_request(message)
-
-        This method is called for each request, and should return
+        """This method is called for each request, and should return
         a reply. The message contains the name of the method to call,
         this function calls that method.
-
         """
         # Get name and args
         name, args, kwargs = message
@@ -747,11 +673,7 @@ class RepChannel(BaseChannel):
             # print 'ack', self._context.id,  package._dest_seq-REQREP_SEQ_REF
 
     def _replier_iteration(self, package):
-        """_replier_iteration()
-
-        Do one iteration: process one request.
-
-        """
+        """Do one iteration: process one request."""
 
         # Get request id
         request_id = package._dest_seq
@@ -767,7 +689,7 @@ class RepChannel(BaseChannel):
 
             # Remove pre-request from pending requests in case of both actions:
             # Cancel pending pre-request, prevent stacking of the same request.
-            for prereq in [prereq for prereq in self._pre_requests]:
+            for prereq in self._pre_requests[:]:
                 if (
                     package._source_id == prereq._source_id
                     and package._dest_seq == prereq._dest_seq
@@ -813,15 +735,11 @@ class RepChannel(BaseChannel):
                     pass  # Channel is closed
                 except Exception:
                     # Probably wrong type of reply returned by handle_request()
-                    print("Warning: request could not be send:")
+                    print("Warning: request could not be sent:")
                     print(getErrorMsg())
 
     def _process_events_local(self, dummy=None):
-        """_process_events_local()
-
-        Called when a message (or more) has been received.
-
-        """
+        """Called when a message (or more) has been received."""
 
         # If closed, unregister from signal and stop the timer
         if self.closed or self._run_mode != 1:
@@ -840,11 +758,8 @@ class RepChannel(BaseChannel):
                 return
 
     def echo(self, arg1, sleep=0.0):
-        """echo(arg1, sleep=0.0)
-
-        Default procedure that can be used for testing. It returns
+        """Default procedure that can be used for testing. It returns
         a tuple (first_arg, context_id)
-
         """
         time.sleep(sleep)
         return arg1, hex(self._context.id)
@@ -867,15 +782,14 @@ class ThreadForReqChannel(threading.Thread):
         # Store channel
         self._channel = channel
 
-        # Make deamon
-        self.setDaemon(True)
+        # Make daemon
+        if sys.version_info < (2, 6):
+            self.setDaemon(True)
+        else:
+            self.daemon = True
 
     def run(self):
-        """run()
-
-        The handler's main loop.
-
-        """
+        """The handler's main loop."""
 
         # Get ref to channel. Remove ref from instance
         channel = self._channel
